#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python version of the pwr R package
@author: Ernest Namdar  ernest.namdar@utoronto.ca
"""
from pypwr.ES_h import ES_h
from pypwr.ES_w1 import ES_w1
from pypwr.ES_w2 import ES_w2
from pypwr.cohen_ES import cohen_ES
from pypwr.pwr_t_test import pwr_t_test
from pypwr.pwr_t2n_test import pwr_t2n_test
from pypwr.pwr_2p_test import pwr_2p_test
from pypwr.pwr_2p2n_test import pwr_2p2n_test
from pypwr.pwr_anova_test import pwr_anova_test
from pypwr.pwr_chisq_test import pwr_chisq_test
from pypwr.pwr_norm_test import pwr_norm_test
from pypwr.pwr_p_test import pwr_p_test
from pypwr.pwr_r_test import pwr_r_test
from pypwr.pwr_f2_test import pwr_f2_test
from pypwr.plot_power_htest import plot_power_htest

name = "pypwr"
